;; -*- mode: scheme; coding: utf-8 -*-

;;;;
;;;; Copyright (C) 2011 - 2015
;;;; David Pirotte <david at altosw dot be>

;;;; This file is part of Grip.
;;;; A Grip of Really Important Procedures.

;;;; Grip is free software: you can redistribute it and/or modify it
;;;; under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.

;;;; Grip is distributed in the hope that it will be useful WARRANTY;
;;;; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
;;;; A PARTICULAR PURPOSE.  See the GNU General Public License for more
;;;; details.

;;;; You should have received a copy of the GNU General Public License
;;;; along with Grip.  If not, see <http://www.gnu.org/licenses/>.
;;;;

;;; Commentary:

;;       <- stands for ->
;; clue                   clutter example
;; clues                  clutter examples set
;; clus                   clutter support

;; There is a bug in clutter 1.12.2 that leads to improper rendering
;; of pango strings.  To 'trick' while waiting for more recent
;; bindings to be implemented, we use a grid and attach a
;; <clutter-text> instance per line of text we need to display, which
;; is correctly rendered when using pango span markup commands.

;;; Code:


(define-module (grip clutter examples hello)
  #:use-module (srfi srfi-1)
  #:use-module (ice-9 receive)
  #:use-module (ice-9 format)
  #:use-module (oop goops)
  #:use-module (gnome-2)
  #:use-module (gnome gobject)
  #:use-module (gnome glib)
  #:use-module (gnome clutter)
  #:use-module (grip clutter colour)
  #:use-module (grip clutter globals)
  #:use-module (grip clutter label)
  #:use-module (grip clutter grid)

  #:export (clue-hello-help-message
	    clue-hello-help-label))


(define %clue-hello-help-message
  "~AThe top left 'Hello World!' is a ~A instance, where as the
center one is a ~A instance, which is gravity sensitive [it
inherits from ~A] and accepts a border size [which was set to
6 for this example].

Right click to change the foreground colour of the ~A instance~A")

(define (clue-hello-help-message)
  (format #f "~?" %clue-hello-help-message
	  (list "<span foreground='Gainsboro'>"
		"<span foreground='#98fb98'>&lt;clutter-text&gt;</span>"
		"<span foreground='#98fb98'>&lt;clus-text&gt;</span>"
		"<span foreground='#98fb98'>&lt;clus-apple&gt;</span>"
		"<span foreground='#98fb98'>&lt;clus-text&gt;</span>"
		"</span>")))

(define (clue-hello-help-label font colour)
  (receive (label lw lh)
      (clus-make-label (clue-hello-help-message) font colour 'use-markup)
    (values label lw lh)))


;;;
;;; Temporary solution to the wrong span rendering problem
;;;


(define text-1 "The top left 'Hello World!' is a ~A instance, where as the")
(define text-2 "center one is a ~A instance, which is gravity sensitive [it")
(define text-3 "inherits from ~A] and accepts a border size [which was set to")
(define text-4 "6 for this example].")
(define text-5 " ")
(define text-6 "Left click to change the ~A instance text and border size [20].")
(define text-7 "Right click to reset and change the foreground colour of the ~A")
(define text-8 "instance.")

(define (clue-hello-fill-grid grid text font colour left top width height)
  ;; (dimfi text)
  (let ((grid-layout (get-layout-manager grid))
	(clutter-text (make <clutter-text> #:text text
			    #:use-markup #t
			    #:font-name font
			    #:color (get-colour colour))))
    (set clutter-text 'x-expand #t)
    (set clutter-text 'y-expand #f)
    (attach grid-layout clutter-text left top width height)))

(define (clue-hello-set-help-grid-size grid)
  (let ((children (get-children grid)))
    (set-size grid
	      (reduce max 0 (map get-width children))
	      (* (length children) (get-height (car children))))))

(define (clue-hello-help-label font bg fg)
  (let ((grid (make <clus-grid> #:row-homogeneous #t
		    #:column-homogeneous #t)))
    (when bg (set-background-color grid (get-colour bg)))
    (clue-hello-fill-grid grid
			  (format #f "~?" text-1
				  (list "<span foreground='#98fb98'>&lt;clutter-text&gt;</span>"))
			  font
			  fg
			  0 0 1 1) ;; left top width height
    (clue-hello-fill-grid grid
			  (format #f "~?" text-2
				  (list "<span foreground='#98fb98'>&lt;clus-text&gt;</span>"))
			  font
			  fg
			  0 1 1 1)
    (clue-hello-fill-grid grid
			  (format #f "~?" text-3
				  (list "<span foreground='#98fb98'>&lt;clus-apple&gt;</span>"))
			  font
			  fg
			  0 2 1 1)
    (clue-hello-fill-grid grid
			  text-4
			  font
			  fg
			  0 3 1 1)
    (clue-hello-fill-grid grid
			  text-5
			  font
			  fg
			  0 4 1 1)
    (clue-hello-fill-grid grid
			  (format #f "~?" text-6
				  (list "<span foreground='#98fb98'>&lt;clus-text&gt;</span>"))
			  font
			  fg
			  0 5 1 1)
   (clue-hello-fill-grid grid
			  (format #f "~?" text-7
				  (list "<span foreground='#98fb98'>&lt;clus-text&gt;</span>"))
			  font
			  fg
			  0 6 1 1)
   (clue-hello-fill-grid grid
			  text-8
			  font
			  fg
			  0 7 1 1)
    (clue-hello-set-help-grid-size grid)
    (values grid (get-width grid) (get-height grid))))
